package edu.buffalo.cse.cse462;

import java.io.IOException;
import java.sql.Date;

import edu.buffalo.cse.cse462.dao.ConnectionInfo;
import edu.buffalo.cse.cse462.dao.CustomerDAO;
import edu.buffalo.cse.cse462.dao.DAOException;
import edu.buffalo.cse.cse462.dao.DAOFactory;
import edu.buffalo.cse.cse462.dao.DAOFactory.Backend;
import edu.buffalo.cse.cse462.dao.DAOSession;
import edu.buffalo.cse.cse462.model.Customer;
import edu.buffalo.cse.cse462.model.impl.CustomerImpl;

public class Driver {

  private String user = null;
  private String passwd = null;
  private DAOSession session = null;

  public static String getLine() throws java.io.IOException {

    String s = "";
    char c = (char) System.in.read();
    while (c != '\n') {
      s += c;
      c = (char) System.in.read();
    }
    return s;
  }

  private boolean readCredentials() {

    user = null;
    passwd = null;
    try {
      System.out.print("Username: ");
      user = getLine();
      System.out.print("Password: ");
      passwd = getLine();
      return user != null && passwd != null;
    }
    catch (IOException e) {
      System.err.println("Could not read user name/password. Stack trace follows:");
      e.printStackTrace();
    }
    return false;
  }

  public void initOracleSession() throws DAOException {

    session = DAOFactory.getDAOSession(Backend.Oracle, ConnectionInfo.Oracle(
        "jdbc:oracle:thin:@aos.acsu.buffalo.edu:1521/aos.buffalo.edu", user, passwd));
  }

//  public void initPostgresSession() throws DAOException {
//
//    session = DAOFactory.getDAOSession(Backend.Postgres,
//        ConnectionInfo.Postgres("localhost", "Sandbox", user, passwd));
//  }

  public void testCustomer() throws DAOException {

    CustomerDAO dao = session.getCustomerDAO();
    Customer joe = new CustomerImpl(1, "Buffalo", Date.valueOf("1980-10-10"), "Joe", "Smith", "NY");
    System.out.println("This is our customer:");
    System.out.println(joe);
    System.out.println("=> Insert customer.");
    dao.insert(joe);
    System.out.println("=> Make insertion permanent.");
    session.commit();
    System.out.println("=> Find recently inserted customer.");
    Customer cid1 = dao.findByKey(1);
    assert(joe.equals(cid1));
    System.out.println(cid1);
    System.out.println("=> Delete recently inserted customer.");
    dao.delete(cid1.getCid());
    System.out.println("=> Find recently deleted customer.");
    Customer deleted = dao.findByKey(1);
    assert(deleted == null);
    System.out.println(deleted);
    System.out.println("=> Abort recent work-- undoes the delete.");
    session.abort();
    System.out.println("=> Find recently inserted customer.");
    cid1 = dao.findByKey(1);
    assert(joe.equals(cid1));
    System.out.println(cid1);
    System.out.println("=> Delete recently inserted customer.");
    dao.delete(cid1.getCid());
    System.out.println("=> This time, make deletion permanent.");
    session.commit();
    System.out.println("=> Find recently deleted customer.");
    deleted = dao.findByKey(1);
    assert(deleted == null);
    System.out.println("=> Should not be here.");
    System.out.println(deleted);
  }


  private void testOrderEntry() {

    // TODO: required implementation for CSE462 Project #1
  }

  private void testOrder() {

    // TODO: required implementation for CSE462 Project #1
  }

  private void testProduct() {

    // TODO: required implementation for CSE462 Project #1
  }

  public static void main(String[] args) throws DAOException {

    Driver d = new Driver();
    if (d.readCredentials()) {
      d.initOracleSession();
      d.testCustomer();
      d.testProduct();
      d.testOrder();
      d.testOrderEntry();
    }
  }
}