package edu.buffalo.cse.cse462.dao;

/**
 * GenericDAO is defines the minimum services implemented by all DAO objects. It is parameterized by
 * the model object M, and by M's key type K.
 * 
 */
public interface GenericDAO<M, K> {

  /**
   * Given an M instance, inserts it into the store. Returns true if the instance was successfully
   * inserted.
   * 
   * @throws DAOException
   */
  public boolean insert(M instance) throws DAOException;

  /**
   * Given the key for an M instance, deletes it from the store. Returns true if the instance was
   * successfully deleted.
   */
  public boolean delete(K key) throws DAOException;

  /**
   * Given the key for an M instance, retrieves the corresponding instance from the store. If one is
   * not found, returns null.
   */
  public M findByKey(K key) throws DAOException;

  /**
   * Given the updated M instance, updates it in the store. This version of update assumes that the
   * key value did not change. Returns true if the instance was successfully updated.
   */
  public boolean update(M updated) throws DAOException;

  /**
   * Given the old key value of an updated M instance, updates it in the store. This version of
   * update assumes that the key value changed. Returns true if the instance was successfully
   * updated.
   */
  public boolean update(K oldKey, M updated) throws DAOException;
}