package edu.buffalo.cse.cse462.internal.dao;

import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

import edu.buffalo.cse.cse462.dao.DAOException;
import edu.buffalo.cse.cse462.dao.DAOSession;

public abstract class JDBCSession implements DAOSession {

  private Connection connection = null;
  private DataSource dataSource = null;

  @Override
  public void abort() throws DAOException {

    if (connection != null) {
      try {
        connection.rollback();
      }
      catch (SQLException e) {
        throw new DAOException("Could not abort work done on the backend.", e);
      }
    }
  }

  @Override
  public void close() throws DAOException {

    if (connection != null) {
      try {
        connection.close();
      }
      catch (SQLException e) {
        throw new DAOException("Could not release connection to the backend.", e);
      }
    }
  }

  @Override
  public void commit() throws DAOException {

    if (connection != null) {
      try {
        connection.commit();
      }
      catch (SQLException e) {
        throw new DAOException("Could not commit work done on the backend.", e);
      }
    }
  }

  public Connection getConnection() throws DAOException {

    if (connection == null && dataSource != null) {
      try {
        connection = dataSource.getConnection();
        connection.setAutoCommit(false);
      }
      catch (SQLException e) {
        throw new DAOException("Could not get a connection to the backend.", e);
      }
    }
    return connection;
  }

  protected DataSource getDataSource() {

    return dataSource;
  }

  protected void setDataSource(DataSource ds) {

    dataSource = ds;
  }
}