package edu.buffalo.cse.cse462.internal.dao.oracle;

import oracle.jdbc.pool.OracleDataSource;

import edu.buffalo.cse.cse462.dao.ConnectionInfo;
import edu.buffalo.cse.cse462.dao.CustomerDAO;
import edu.buffalo.cse.cse462.dao.DAOException;
import edu.buffalo.cse.cse462.dao.OrderDAO;
import edu.buffalo.cse.cse462.dao.OrderEntryDAO;
import edu.buffalo.cse.cse462.dao.ProductDAO;
import edu.buffalo.cse.cse462.internal.dao.JDBCSession;

public final class OracleDAOSession extends JDBCSession {

  public static final OracleDAOSession INSTANCE = new OracleDAOSession();
  
  // block direct creation
  private OracleDAOSession() {
    
  }

  /*
   * Configure a physical connection to the Oracle database via JDBC.
   */
  public static OracleDAOSession create(ConnectionInfo ci) throws DAOException {

    if (INSTANCE.getDataSource() == null) {
      try {
        OracleDataSource ds = new OracleDataSource();
        ds.setUser(ci.getUser());
        ds.setPassword(ci.getPassword());
        ds.setURL(ci.getUrl());
        INSTANCE.setDataSource(ds);
      }
      catch (Exception e) {
        throw new DAOException("Could not create data source.", e);
      }
    }
    return INSTANCE;
  }

  @Override
  public CustomerDAO getCustomerDAO() {

    return new OracleCustomerDAO(INSTANCE);
  }

  @Override
  public OrderDAO getOrderDAO() {

    return new OracleOrderDAO(INSTANCE);
  }

  @Override
  public OrderEntryDAO getOrderEntryDAO() {

    return new OracleOrderEntryDAO(INSTANCE);
  }

  @Override
  public ProductDAO getProductDAO() {

    return new OracleProductDAO(INSTANCE);
  }
}