package edu.buffalo.cse.cse462.model.impl;

import java.sql.Date;

import edu.buffalo.cse.cse462.model.Customer;

/**
 * Implementation of the Customer model object.
 * 
 * @author Demian Lessa
 */
public class CustomerImpl implements Customer {

  private final Integer cid;
  private final Date dateOfBirth;
  private final String city;
  private final String firstName;
  private final String lastName;
  private final String state;

  public CustomerImpl(Integer cid, String city, Date dateOfBirth, String firstName,
      String lastName, String state) {

    this.cid = cid;
    this.dateOfBirth = dateOfBirth;
    this.city = city;
    this.state = state;
    this.firstName = firstName;
    this.lastName = lastName;
  }

  @Override
  public Integer getCid() {

    return cid;
  }

  @Override
  public String getCity() {

    return city;
  }

  @Override
  public Date getDateOfBirth() {

    return dateOfBirth;
  }

  @Override
  public String getFirstName() {

    return firstName;
  }

  @Override
  public String getLastName() {

    return lastName;
  }

  @Override
  public String getState() {

    return state;
  }

  @Override
  public int hashCode() {

    int hash = 7;
    hash = hash * 31 + getCid().hashCode();
    hash = hash * 31 + getCity().hashCode();
    hash = hash * 31 + getFirstName().hashCode();
    hash = hash * 31 + getState().hashCode();
    hash = hash * 31 + getLastName().hashCode();
    hash = hash * 31 + (getDateOfBirth() == null ? 0 : getDateOfBirth().hashCode());
    return hash;
  }

  @Override
  public boolean equals(Object other) {

    // obvious, identity
    if (this == other) {
      return true;
    }
    // obvious, incompatible
    if (!(other instanceof Customer)) {
      return false;
    }
    // equality based on actual data values
    Customer customer = (Customer) other;
    return getCid().equals(customer.getCid())
        && getCity().equals(customer.getCity())
        && getState().equals(customer.getState())
        && getFirstName().equals(customer.getFirstName())
        && getLastName().equals(customer.getLastName())
        && ((getDateOfBirth() == null && customer.getDateOfBirth() == null) || (getDateOfBirth() != null && getDateOfBirth()
            .equals(customer.getDateOfBirth())));
  }

  @Override
  public String toString() {

    return String.format("[cid: %d] %s, %s was born on %s and lives in %s-%s", cid, lastName,
        firstName, dateOfBirth, city, state);
  }
}